@@ -2,6 +2,8 @@
     maybe_uninit_write_slice,
     process_exitcode_placeholder,
     result_into_ok_or_err,
+    slice_as_chunks,
+    new_uninit,
     vec_into_raw_parts
 )]
 
@@ -46,16 +48,16 @@ fn output_directory<'a>(
 
 
 fn output_file_name(
-    space: &dyn spaces::Space,
+    space: &spaces::Space,
     out_dir: &std::path::Path,
     file_stem: &std::ffi::OsStr,
 ) -> std::path::PathBuf {
     let bytes = std::os::unix::ffi::OsStrExt::as_bytes(file_stem);
-    let suffix = space.get_file_suffix();
+    let suffix = space.name;
     let mut buf = Vec::<u8>::with_capacity(bytes.len() + suffix.len() + 6);
     buf.extend_from_slice(bytes);
     buf.push(b'-');
-    buf.extend_from_slice(suffix);
+    buf.extend_from_slice(suffix.as_bytes());
     buf.extend_from_slice(b".webp");
     let file_name: std::ffi::OsString =
         std::os::unix::ffi::OsStringExt::from_vec(buf);
@@ -88,19 +90,19 @@ fn process_file(
     } else {
         return false;
     };
-    let errors = spaces::SPACES
+    let errors = opts
+        .spaces
         .par_iter()
-        .filter(|&space| {
+        .filter(|space| {
             let out_file =
-                output_file_name(*space, out_dir.as_ref(), file_stem);
+                output_file_name(space.0, out_dir.as_ref(), file_stem);
             if !confirmer.confirm(&out_file) {
                 return true;
             }
             eprintln!("Generating {}...", out_file.to_string_lossy());
-            let img = space.build_image(&img);
-            let enc = opts.encode(webp::Encoder::from_image(
-                &image::DynamicImage::ImageRgb8(img),
-            ));
+            let (width, height, img) = spaces::build_image(space.0, &img);
+            let enc =
+                opts.encode(webp::Encoder::from_rgb(&img[..], width, height));
             if let Err(err) = std::fs::File::create(&out_file)
                 .and_then(|mut fd| fd.write_all(&enc))
             {
@@ -115,13 +117,21 @@ fn process_file(
 }
 
 fn main() -> std::process::ExitCode {
-    let opts = <cli::Opts as clap::Clap>::parse();
+    let mut opts = <cli::Opts as clap::Clap>::parse();
     if let Some(dir) = &opts.out_dir {
         if let Err(err) = std::fs::create_dir_all(dir) {
             perr!(dir, err);
             return std::process::ExitCode::FAILURE;
         }
     }
+    if opts.spaces.is_empty() {
+        opts.spaces.extend(spaces::SPACES.iter().map(cli::SpaceArg));
+    } else {
+        opts.spaces
+            .sort_by_key(|space| space.0 as *const _ as usize);
+        opts.spaces.dedup_by_key(|space| space.0 as *const _);
+    }
+    let opts = opts;
     if let Some(num) = opts.jobs {
         rayon::ThreadPoolBuilder::new()
             .num_threads(num.max(1))
